# This file is part of Grow-bots.
#
# Grow-bots is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Grow-bots is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Grow-bots.  If not, see <http://www.gnu.org/licenses/>.

import random
import pygame
from OpenGL.GL import *
from OpenGL.GLU import *

import dyna
import load

"""
Adapted from:
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/115418
"""

def fix_alpha(data):
    out = []
    for i in xrange(0, len(data), 4):
        for j in (0, 1, 2, 0):
            out += data[i + j]
    return "".join(out)


class Gui:

    keys = {
        '`' :'~', '1' :'!', '2':'@', '3':'#', '4' :'$', '5' :'%', '6':'^',
        '7':'&',  '8' :'*', '9' :'(', '0':')', '-':'_', '=' :'+', 'q' :'Q',
        'w':'W', 'e':'E', 'r' :'R', 't' :'T', 'y':'Y', 'u':'U', 'i' :'I',
        'o' :'O', 'p':'P', '[':'{', ']' :'}', '\\':'|', 'a':'A', 's':'S',
        'd' :'D', 'f' :'F', 'g':'G', 'h':'H', 'j' :'J', 'k' :'K', 'l':'L',
        ';':':', '\'':'"', 'z' :'Z', 'x':'X', 'c':'C', 'v' :'V', 'b' :'B',
        'n':'N', 'm':'M', ',' :'<', '.' :'>', '/':'?'
    }

    def __init__(self, font, color, bgcolor):
        self.digits = {}
        self.cache = {}
        self.key_collect = []
        self.update_font(font, color, bgcolor)

    def update_font(self, font, color, bgcolor):
        self.font = font
        self.color = color
        self.bgcolor = bgcolor
        self.char_w, self.char_h = 0, 0
        for i in xrange(1, 256):
            c = chr(i)
            img = self.font.render(c, 1, color, bgcolor)
            data = fix_alpha(pygame.image.tostring(img, "RGBA", True))
            w, h = img.get_size()
            self.digits[c] = (w, h, data)
            self.char_w = max(self.char_w, w)
            self.char_h = max(self.char_h, h)
        self.width = dyna.getcontext().width / self.char_w
        self.height = dyna.getcontext().height / self.char_h

    def collect(self, key):
        self.key_collect.append(key)

    def simplewrite(self, x, y, value):
        x = x * self.char_w
        y = y * self.char_h
        glEnable(GL_ALPHA_TEST)
        for c in value:
            glRasterPos2i(x, y)
            ch = self.digits[c]
            glDrawPixels(ch[0], ch[1], GL_RGBA, GL_UNSIGNED_BYTE, ch[2])
            x += self.char_w
        glDisable(GL_ALPHA_TEST)
        return x

    def write(self, x, y, values):
        glEnable(GL_ALPHA_TEST)
        if y < 0:
            y = (dyna.getcontext().height / self.char_h) + (y - 1)
        x, y = x * self.char_w, (y+1) * self.char_h
        for value in values:
            if isinstance(value, int) or isinstance(value, long):
                for c in str(value):
                    glRasterPos2i(x, y)
                    ch = self.digits[c]
                    glDrawPixels(ch[0], ch[1], GL_RGBA, GL_UNSIGNED_BYTE, ch[2])
                    x += self.char_w
            else:
                if value not in self.cache:
                    img = self.font.render(value, 1, self.color, self.bgcolor)
                    data = fix_alpha(pygame.image.tostring(img, "RGBA", True))
                    w, h = img.get_size()
                    self.cache[value] = (w, h, data)
                ch = self.cache[value]
                glRasterPos2i(x, y)
                glDrawPixels(ch[0], ch[1], GL_RGBA, GL_UNSIGNED_BYTE, ch[2])
                x += ch[0]
        glDisable(GL_ALPHA_TEST)

    def alert(self, values, padding=2):
        if isinstance(values, str):
            values = [values]
        w = max(len(text) for text in values)
        h = len(values) - 1
        x = (self.width / 2) - (w / 2)
        y = (self.height / 2) - (h / 2)
        dyn = dyna.getcontext()
        is_open = True
        dyn.events.add("keydown", self.collect)
        while is_open:
            self.key_collect = []
            dyn.process()
            dyn.render(flip=False)
            dyn.push2d()
            glPushMatrix()
            glScalef(self.char_w, self.char_h, 1.0)
            glBegin(GL_QUADS)
            glColor3ubv(self.bgcolor)
            glVertex2f(x-padding, y-padding-1)
            glVertex2f(x+padding+w, y-padding-1)
            glVertex2f(x+padding+w, y+padding+h)
            glVertex2f(x-padding, y+padding+h)
            glEnd()
            glPopMatrix()
            for i, text in enumerate(values):
                self.simplewrite(x, y + i, text)
            dyn.pop2d()
            pygame.display.flip()
            for key in self.key_collect:
                if key == pygame.K_RETURN:
                    is_open = False
        dyn.events.remove("keydown", self.collect)
        dyn.keys.clear()

    def prompt(self, value, size=10, padding=2):
        start = len(value)
        value = list(value)
        w = start + size + 1
        x = (self.width / 2) - (w / 2)
        y = self.height / 2
        dyn = dyna.getcontext()
        inkey = dyn.keys.is_down
        onkey = dyn.keys.on_down
        current = []
        is_open = True
        pygame.key.set_repeat(200, 50)
        dyn.events.add("keydown", self.collect)
        while is_open:
            self.key_collect = []
            dyn.process()
            in_shft = inkey(pygame.K_LSHIFT) or inkey(pygame.K_RSHIFT)
            for key in self.key_collect:
                if key>=0 and key <=255 and chr(key) in Gui.keys:
                    current.append(Gui.keys[chr(key)] if in_shft else chr(key))
                elif key == pygame.K_SPACE:
                    current.append(" ")
                elif key == pygame.K_BACKSPACE:
                    current = current[:-1]
                elif key == pygame.K_RETURN:
                    is_open = False
            if len(current) > size:
                current = current[:size]
            dyn.render(flip=False)
            dyn.push2d()
            glPushMatrix()
            glScalef(self.char_w, self.char_h, 1.0)
            glBegin(GL_QUADS)
            glColor3ubv(self.bgcolor)
            glVertex2f(x-padding, y-padding-1)
            glVertex2f(x+padding+w, y-padding-1)
            glVertex2f(x+padding+w, y+padding)
            glVertex2f(x-padding, y+padding)
            glEnd()

            glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
            glBegin(GL_QUADS)
            glColor3ub(0, 0, 0)
            glVertex2f(x+start, y-1)
            glVertex2f(x+start+size+1, y-1)
            glVertex2f(x+start+size+1, y)
            glVertex2f(x+start, y)
            glEnd()
            glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)

            glPopMatrix()
            self.simplewrite(x, y, value + current + ["_"])
            dyn.pop2d()
            pygame.display.flip()
        dyn.events.remove("keydown", self.collect)
        pygame.key.set_repeat(0, 0)
        dyn.keys.clear()
        return "".join(current)

